<?php
// All code is based on the (US) Federal Meteorological Handbook
// http://www.ofcm.gov/fmh-1/fmh1.htm

class WeatherIconMetar
{
	var $decoded_metar;
	var $gmt_offset;
	var $before;
	var $after;
	
	function WeatherIconMetar($metar, $latitude, $longitude, $gmt_offset, $dst=TRUE, $before='', $after='')
	{
		// clear the decoded metar 
		$this->decoded_metar			= array();
		$this->decoded_metar['metar']   = $metar;
		$this->decoded_metar['remarks'] = '';

		// Calculate sunrise/sunset and insert into decoded_metar
		$this->sunrise_sunset($latitude, $longitude, $gmt_offset, $dst);
		
		// Save the other parameters
		$this->gmt_offset			  = $gmt_offset;
		$this->before				  = $before;
		$this->after				  = $after;
		
		// Decode our metar
		$this->decoded_metar();
	}
	
	function store_speed($value, $windunit, &$knots, &$meterspersec, &$kilometersperhour, &$milesperhour) 
	{
		if ($value == 0) 
		{
			$knots = 0;
			$meterspersec = 0;
			$kilometersperhour = 0;
			$milesperhour = 0;
			return;
		}
		
		if ($windunit == 'KT') 
		{
			// The windspeed measured in knots:
			$knots		= number_format($value);
			
			// The windspeed measured in meters per second, rounded to one decimal place
			$meterspersec = number_format($value * 0.5144, 1);
			
			// The windspeed measured in kilometers per hour, rounded to one decimal place
			$kilometersperhour = number_format($value * 1.85200, 1);
			
			// The windspeed measured in miles per hour, rounded to one decimal place
			$milesperhour = number_format($value * 1.1508, 1);			
		}
		elseif ($windunit == 'MPS') 
		{
			// The windspeed measured in meters per second
			$meterspersec = number_format($value);
			
			// The windspeed measured in knots, rounded to one decimal place
			$knots		= number_format($value / 0.5144, 1);

			// The windspeed measured in kilometers per hour, rounded to one decimal place
			$kilometersperhour = number_format($value * 3.6, 1);
			
			// The windspeed measured in miles per hour, rounded to one decimal place
			$milesperhour = number_format($value / 0.5144 * 1.1508, 1);
		}
		elseif ($windunit == 'KMH')
		{
			// The windspeed measured in kilometers per hour
			$kilometersperhour = number_format($value);
						
			// The windspeed measured in kilometers per hour
			$meterspersec = number_format($value * 1000 / 3600, 1);
			$knots		= number_format($value * 1000 / 3600 / 0.5144, 1);
			
			// The windspeed measured in miles per hour, rounded to one decimal place
			$milesperhour = number_format($knots * 1.1508, 1);
		}
	}
	
	function store_temp($temp, &$temp_c, &$temp_f) 
	{
		// Note: $temp is converted to negative if $temp > 100.0 (See
		// Federal Meteorological Handbook for groups T, 1, 2 and 4). 
		// For example, a temperature of 2.6C and dew point of -1.5C 
		// would be reported in the body of the report as "03/M01" and the
		// TsnT'T'T'snT'dT'dT'd group as "T00261015").
	
		if ($temp[0] == 1)
		{
			$temp[0] = '-';
		}
		
		// The temperature in Celsius.
		$temp_c = number_format($temp,1);
		
		// The temperature in Fahrenheit.
		$temp_f = number_format($temp * (9/5) + 32, 1);
	}
	
	function decode_direction($deg)
	{
		$dir = '';
		
		switch(round($deg / 22.5) % 16)
		{
			case 0:  $dir = __('N',   'WI'); break;
			case 1:  $dir = __('NNE', 'WI'); break;
			case 2:  $dir = __('NE',  'WI'); break;
			case 3:  $dir = __('ENE', 'WI'); break;
			case 4:  $dir = __('E',   'WI'); break;
			case 5:  $dir = __('ESE', 'WI'); break;
			case 6:  $dir = __('SE',  'WI'); break;
			case 7:  $dir = __('SSE', 'WI'); break;
			case 8:  $dir = __('S',   'WI'); break;
			case 9:  $dir = __('SSW', 'WI'); break;
			case 10: $dir = __('SW',  'WI'); break;
			case 11: $dir = __('WSW', 'WI'); break;
			case 12: $dir = __('W',   'WI'); break;
			case 13: $dir = __('WNW', 'WI'); break;
			case 14: $dir = __('NW',  'WI'); break;
			case 15: $dir = __('NNW', 'WI'); break;
		}
		
		return $dir;
	}
	
	function decoded_metar()
	{
		global $WI;

		// initialization
		$temp_visibility_miles = '';

		// Parse the metar
		$parts = explode(' ', $this->decoded_metar['metar']);
		$num_parts = count($parts);

		for ($i = 0; $i < $num_parts; $i++) 
		{
			$part = $parts[$i];
			
			if (ereg('RMK|TEMPO|BECMG|INTER', $part)) 
			{
				// The rest of the METAR is either a remark or temporary
				// information. We keep the remark.
				for($j=$i;$j<$num_parts; $j++)
				{
					$this->decoded_metar['remarks'] .= ' ' . $parts[$j];
				}
				
				$this->decoded_metar['remarks'] = trim($this->decoded_metar['remarks']);
				break;
			}
			elseif ($part == 'METAR') 
			{
				// Type of Report: METAR
				$this->decoded_metar['type'] = 'METAR';
			}
			elseif ($part == 'SPECI')
			{
				// Type of Report: SPECI
				$this->decoded_metar['type'] = 'SPECI';
			}
			elseif (ereg('^[A-Z]{4}$', $part) && empty($this->decoded_metar['icao']))
			{
				// Station Identifier
				$this->decoded_metar['icao'] = $part;
			}
			elseif (ereg('([0-9]{2})([0-9]{2})([0-9]{2})Z', $part, $regs))
			{
				// Date and Time of Report.
				//
				// We return a standard Unix UTC/GMT timestamp suitable for
				// gmdate().
				//
				// If you experience incorrect timestamps but cannot set the
				// clock, then you can set $this->properties['offset'] to be
				// the offset in hours to add. For example, if your times
				// generated are 1 hour too early (so METARs appear an hour
				// older than they are), set $this->properties['offset'] to be
				// +1 in your defaults.php file.
				if ($regs[1] > gmdate('j'))
				{
					// The day is greather that the current day of month => the
					// report is from last month. 
					$month = gmdate('n') - 1;
				}
				else
				{
					$month = gmdate('n');
				}

				$this->decoded_metar['time']['update'] = 
				gmmktime($regs[2], $regs[3], 0, $month, $regs[1], gmdate('Y'));
			}
			elseif (ereg('(AUTO|COR|RTD|CC[A-Z]|RR[A-Z])', $part, $regs))
			{
				// Report Modifier: AUTO, COR, CCx or RRx
				$this->decoded_metar['report_mod'] = $regs[1];
			}
			elseif (ereg('([0-9]{3}|VRB)([0-9]{2,3})G?([0-9]{2,3})?(KT|MPS|KMH)', $part, $regs)) 
			{
				// Wind Group
				$this->decoded_metar['wind']['deg'] = $regs[1];
				$this->decoded_metar['wind']['dir'] = $this->decode_direction($regs[1]);
				
				$this->store_speed($regs[2],
								   $regs[4],
								   $this->decoded_metar['wind']['knots'],
								   $this->decoded_metar['wind']['m/s'],
								   $this->decoded_metar['wind']['km/h'],
								   $this->decoded_metar['wind']['mph']);

				if (!empty($regs[3]))
				{
					// We have a report with information about the gust.
					// First we have the gust measured in knots.
					$this->store_speed($regs[3],
									   $regs[4],
									   $this->decoded_metar['wind']['gust']['knots'],
									   $this->decoded_metar['wind']['gust']['m/s'],
									   $this->decoded_metar['wind']['gust']['km/h'],
									   $this->decoded_metar['wind']['gust']['mph']);
				}
			}
			elseif (ereg('^([0-9]{3})V([0-9]{3})$', $part, $regs) &&
					!empty($this->decoded_metar['wind']))
			{
				// Variable wind-direction
				$this->decoded_metar['wind']['var_beg'] = $regs[1];
				$this->decoded_metar['wind']['var_end'] = $regs[2];
			}
			elseif (ereg('^([0-9]{4})([NESW]?[NESWD]?[NESWV]?)$', $part, $regs))
			{
				// Visibility in meters (4 digits only)
				unset($group);

				if ($regs[1] == '0000') 
				{
					// Special low value
					
					$group['prefix']	= -1;	 //
					$group['m']			= 50;
					$group['km']		= 0.05;
					$group['ft']		= 164;
					$group['miles']		= 0.031;
				}
				elseif ($regs[1] == '9999')
				{
					// Special high value
					$group['prefix']	= 1; 
					$group['m']			= 10000;
					$group['km']		= 10;
					$group['ft']		= 32800;
					$group['miles']		= 6.2;
				}
				else
				{
					// Normal visibility, returned in both small and large units.
					$group['prefix'] = 0; 
					$group['km']	 = number_format($regs[1]/1000, 1);
					$group['miles']  = number_format($regs[1]/1609.344, 1);
					$group['m'] = $regs[1] * 1;
					$group['ft']	 = round($regs[1] * 3.28084);
				}
				
				if (!empty($regs[2])) 
				{
					$group['deg'] = $regs[2];
				}
				
				$this->decoded_metar['visibility'][] = $group;

			}
			elseif (ereg('^[0-9]$', $part))
			{
				// Temp Visibility Group, single digit followed by space.
				$temp_visibility_miles = $part;
			}
			elseif (ereg('^M?(([0-9]?)[ ]?([0-9])(/?)([0-9]*))SM$', 
					$temp_visibility_miles . ' ' . $part, $regs))
			{
				// Visibility Group
				unset($group);
				
				if ($regs[4] == '/')
				{
					$vis_miles = $regs[2] + $regs[3]/$regs[5];
				}
				else
				{
					$vis_miles = $regs[1];
				}
				
				if ($regs[0][0] == 'M')
				{
					// Prefix - less than
					$group['prefix'] = -1;
				}
				else
				{
					$group['prefix'] = 0;
				}

				// The visibility measured in miles
				$group['miles']  = number_format($vis_miles, 1);

				// The visibility measured in feet
				$group['ft']	 = round($vis_miles * 5280, 1);

				// The visibility measured in kilometers
				$group['km']	 = number_format($vis_miles * 1.6093, 1);

				// The visibility measured in meters
				$group['m'] = round($vis_miles * 1609.3);

				$this->decoded_metar['visibility'][] = $group;
			}
			elseif ($part == 'CAVOK')
			{
				// CAVOK is used when the visibility is greater than 10
				// kilometers, the lowest cloud-base is at 5000 feet or more
				// and there is no significant weather.
				unset($group);
				$group['prefix'] = 1; 
				$group['km']	 = 10;
				$group['m'] = 10000;
				$group['miles']  = 6.2;
				$group['ft']	 = 32800;
				$this->decoded_metar['visibility'][] = $group;
				$this->decoded_metar['clouds'][]['condition'] = 'CAVOK';

			}
			elseif (ereg('^R([0-9]{2})([RLC]?)/([MP]?)([0-9]{4})' .
						 '([DNU]?)(FT?)?V?(P?)([0-9]{4})?([DNU]?)?(FT?)?/?([NWSE]?)?$', $part, $regs)) 
			{
				// Runway-group
				unset($group);
				$group['nr'] = $regs[1];

				if (!empty($regs[2]))
				{
					$group['approach'] = $regs[2];
				}

				if (!empty($regs[8]))
				{
					// We have both min and max visibility since $regs[7] holds
					// the max visibility.
					if (!empty($regs[5]))
					{ 
						// $regs[5] is tendency for min visibility.
						$group['min_tendency'] = $regs[5];
					}

					if (!empty($regs[9])) 
					{				 
						// $regs[9] is tendency for max visibility.
						$group['max_tendency'] = $regs[9];
					}

					if ($regs[3] == 'M')
					{
						// Less than.
						$group['min_prefix'] = -1;
					}
				
					if ($regs[6] == 'FT')
					{
						$group['min_meter'] = round($regs[4] / 3.2808);
						$group['min_ft']	= $regs[4] * 1;
					}
					else
					{
						$group['min_meter'] = $regs[4] * 1;
						$group['min_ft']	= round($regs[4] * 3.2808);
					}

					if ($regs[7] == 'P')
					{
						// Greater than.
						$group['max_prefix'] = 1;
					}

					if ($regs[10] == 'FT')
					{
						$group['max_meter'] = $regs[8] * 1;
						$group['max_ft']	= round($regs[8] * 3.2808);
					}
					else
					{
						$group['max_ft']	= $regs[8] * 1;
						$group['max_meter'] = round($regs[8] / 3.2808);
					}

				}
				else
				{
					// We only have a single visibility.

					if (!empty($regs[5]))
					{ 
						// $regs[5] holds the tendency for visibility.
						$group['tendency'] = $regs[5];
					}

					if ($regs[3] == 'M') 
					{
						// Less than.
						$group['prefix'] = -1;
					}
					elseif ($regs[3] == 'P') 
					{
						// Greater than.
						$group['prefix'] = 1;
					}

					if ($regs[6] == 'FT')
					{
						$group['m'] = round($regs[4] / 3.2808);
						$group['ft']	 = $regs[4] * 1;
					}
					else
					{
						$group['m'] = $regs[4] * 1;
						$group['ft']	 = round($regs[4] * 3.2808);
					}

					if (!empty($regs[11]))
					{
						$group['deg'] = $regs[11];
					}
				}
				
				$this->decoded_metar['runway'][] = $group;

			}
			elseif (ereg('^(VC)?' .						   // Proximity
						 '(-|\+)?' .						  // Intensity
						 '(MI|PR|BC|DR|BL|SH|TS|FZ)?' .	   // Descriptor
						 '((DZ|RA|SN|SG|IC|PL|GR|GS|UP)+)?' . // Precipitation
						 '(BR|FG|FU|VA|DU|SA|HZ|PY)?' .		  // Obscuration
						 '(PO|SQ|FC|SS)?$',					  // Other
						 $part, $regs))
			{
				// Current weather-group.
				$this->decoded_metar['weather'][] = array('proximity'	 => $regs[1],
													'intensity'	 => $regs[2],
													'descriptor'	=> $regs[3],
													'precipitation' => $regs[4],
													'obscuration'   => $regs[6],
													'other'		 => $regs[7]);
			}
			elseif ($part == 'SKC' || $part == 'CLR' || $part == 'NSC') 
			{
				// Cloud-group
				$this->decoded_metar['clouds'][]['condition'] = $part;
			}
			elseif (ereg('^(VV|FEW|SCT|BKN|OVC)([0-9]{3}|///)' .
						 '(CB|TCU)?$', $part, $regs))
			{
				// We have found (another) a cloud-layer-group.
				unset($group);

				$group['condition'] = $regs[1];
				if (!empty($regs[3]))
				{
					$group['cumulus'] = $regs[3];
				}
				
				if ($regs[2] == '000') 
				{
					// '000' is a special height.
					$group['km']		= 0.03;
					$group['miles']		= 0.02;
					$group['m']			= 30;
					$group['ft']		= 100;
					$group['prefix']	= -1; // Less than
				}
				elseif ($regs[2] == '///')
				{
					// '///' means height nil
					$group['km']	= 'nil';
					$group['miles']	= 'nil';
					$group['m']		= 'nil';
					$group['ft']	= 'nil';

				}
				else
				{
					$group['km']	= number_format($regs[2] * .03048, 2);
					$group['miles']	= number_format($regs[2] * .0189, 2);
					$group['m']		= round($regs[2] * 30.48);
					$group['ft']	= $regs[2] *100;
				}
				$this->decoded_metar['clouds'][] = $group;

			}
			elseif (ereg('^(M?[0-9]{2})/(M?[0-9]{2}|//)?$', $part, $regs))
			{
				// Temperature/Dew Point Group.
				$this->decoded_metar['temperature']['temp_c'] =
					round(strtr($regs[1], 'M', '-'));
				
				$this->decoded_metar['temperature']['temp_f'] =
					round(strtr($regs[1], 'M', '-') * (9/5) + 32);

				// The dewpoint could be missing, this is indicated by the
				// second group being empty at most places, but in the UK they
				// use '//' instead of the missing temperature...
				if (!empty($regs[2]) && $regs[2] != '//')
				{
					$this->decoded_metar['temperature']['dew_c'] =
						round(strtr($regs[2], 'M', '-'));
					$this->decoded_metar['temperature']['dew_f'] =
						round(strtr($regs[2], 'M', '-') * (9/5) + 32);
				}
			}
			elseif (ereg('A([0-9]{4})', $part, $regs))
			{
				// Altimeter.
				// The pressure measured in inHg.
				$this->decoded_metar['altimeter']['inhg'] =
					number_format($regs[1]/100, 2);

				// The pressure measured in mmHg, hPa and atm
				$this->decoded_metar['altimeter']['mmhg'] =
					number_format($regs[1] * 0.254, 1, '.', '');
					
				$this->decoded_metar['altimeter']['hpa']  =
					round($regs[1] * 0.33864);
					
				$this->decoded_metar['altimeter']['atm']  =
					number_format($regs[1] * 3.3421e-4, 3, '.', '');
					
			}
			elseif (ereg('Q([0-9]{4})', $part, $regs))
			{
				// Altimeter.
				// The specification doesn't say anything about
				// the Qxxxx-form, but it's in the METARs.

				// The pressure measured in hPa
				$this->decoded_metar['altimeter']['hpa']  = round($regs[1]);
				
				// The pressure measured in mmHg, inHg and atm
				$this->decoded_metar['altimeter']['mmhg'] =
					number_format($regs[1] * 0.75006, 1, '.', '');
				
				$this->decoded_metar['altimeter']['inhg'] =
					number_format($regs[1] * 0.02953, 2);
				
				$this->decoded_metar['altimeter']['atm']  =
					number_format($regs[1] * 9.8692e-4, 3, '.', '');
			}
			elseif (ereg('^T([0-9]{4})([0-9]{4})', $part, $regs))
			{
				// Temperature/Dew Point Group, coded to tenth of degree Celsius.
				$this->store_temp($regs[1] / 10,
								  $this->decoded_metar['temperature']['temp_c'],
								  $this->decoded_metar['temperature']['temp_f']);
				
				$this->store_temp($regs[2] / 10,
								  $this->decoded_metar['temperature']['dew_c'],
								  $this->decoded_metar['temperature']['dew_f']);
			}
			elseif (ereg('^T([0-9]{4}$)', $part, $regs)) 
			{
				$this->store_temp($regs[1],
								  $this->decoded_metar['temperature']['temp_c'],
								  $this->decoded_metar['temperature']['temp_f']);
			}
			elseif (ereg('^1([0-9]{4}$)', $part, $regs))
			{
				// 6 hour maximum temperature Celsius, coded to tenth of degree
				$this->store_temp($regs[1] / 10,
								  $this->decoded_metar['temp_min_max']['max6h_c'],
								  $this->decoded_metar['temp_min_max']['max6h_f']);
			}
			elseif (ereg('^2([0-9]{4}$)', $part, $regs)) 
			{
				// 6 hour minimum temperature Celsius, coded to tenth of degree
				$this->store_temp($regs[1] / 10,
								  $this->decoded_metar['temp_min_max']['min6h_c'],
								  $this->decoded_metar['temp_min_max']['min6h_f']);
			}
			elseif (ereg('^4([0-9]{4})([0-9]{4})$', $part, $regs))
			{
				// 24 hour maximum and minimum temperature Celsius, coded to
				// tenth of degree
				$this->store_temp($regs[1] / 10,
								  $this->decoded_metar['temp_min_max']['max24h_c'],
								  $this->decoded_metar['temp_min_max']['max24h_f']);
						   
				$this->store_temp($regs[2] / 10,
								  $this->decoded_metar['temp_min_max']['min24h_c'],
								  $this->decoded_metar['temp_min_max']['min24h_f']);
			}
			elseif (ereg('^P([0-9]{4})', $part, $regs))
			{
				// Precipitation during last hour in hundredths of an inch
				if ($regs[1] == '0000')
				{
					$this->decoded_metar['precipitation']['in'] = -1;
					$this->decoded_metar['precipitation']['mm'] = -1;
				}
				else
				{
					$this->decoded_metar['precipitation']['in'] =
						number_format($regs[1]/100, 2);
					$this->decoded_metar['precipitation']['mm'] =
						number_format($regs[1]*0.254, 2);
				}
			}
			elseif (ereg('^6([0-9]{4})', $part, $regs))
			{
				// Precipitation during last 3 or 6 hours in hundredths of an
				// inch.
				if ($regs[1] == '0000')
				{
					$this->decoded_metar['precipitation']['in_6h'] = -1;
					$this->decoded_metar['precipitation']['mm_6h'] = -1;
				}
				else
				{
					$this->decoded_metar['precipitation']['in_6h'] =
						number_format($regs[1]/100, 2);
						
					$this->decoded_metar['precipitation']['mm_6h'] =
						number_format($regs[1]*0.254, 2);
				}
			}
			elseif (ereg('^7([0-9]{4})', $part, $regs))
			{
				// Precipitation during last 24 hours in hundredths of an inch.
				if ($regs[1] == '0000')
				{
					$this->decoded_metar['precipitation']['in_24h'] = -1;
					$this->decoded_metar['precipitation']['mm_24h'] = -1;
				}
				else
				{
					$this->decoded_metar['precipitation']['in_24h'] =
						number_format($regs[1]/100, 2, '.', '');
					$this->decoded_metar['precipitation']['mm_24h'] =
						number_format($regs[1]*0.254, 2, '.', '');
				}
			}
			elseif (ereg('^4/([0-9]{3})', $part, $regs))
			{
				// Snow depth in inches
				if ($regs[1] == '0000')
				{
					$this->decoded_metar['precipitation']['snow_in'] = -1;
					$this->decoded_metar['precipitation']['snow_mm'] = -1;
				}
				else
				{
					$this->decoded_metar['precipitation']['snow_in'] = $regs[1] * 1;
					$this->decoded_metar['precipitation']['snow_mm'] = round($regs[1] * 25.4);
				}
			}
			else
			{
				 // If we couldn't match the group, we assume that it was a
				 // remark.
				$this->decoded_metar['remarks'] .= ' ' . $part;
			}
		}
		
		// ---------------------
		// POST METAR PROCESSING
		// ---------------------

		// Relative humidity
		if (!empty($this->decoded_metar['temperature']['temp_c']) &&
			!empty($this->decoded_metar['temperature']['dew_c']))
		{
			$WI->status[] = $WI->gmtime() . " Computing Relative Humidty.";
			$this->decoded_metar['rel_humidity'] =
				number_format(pow(10, (1779.75 * ($this->decoded_metar['temperature']['dew_c']
												 - $this->decoded_metar['temperature']['temp_c'])
									  / ((237.3 + $this->decoded_metar['temperature']['dew_c'])
									  * (237.3 + $this->decoded_metar['temperature']['temp_c']))
									  + 2)), 1);
		} 

		// Compute windchill if temp < 51f and windspeed > 3 mph
		if (!empty($this->decoded_metar['temperature']['temp_f']) && 
			$this->decoded_metar['temperature']['temp_f'] < 51 &&
			!empty($this->decoded_metar['wind']['mph']) &&
			$this->decoded_metar['wind']['mph'] > 3) 
		{
			$WI->status[] = $WI->gmtime() . " Computing WindChill.";
			$this->decoded_metar['windchill']['windchill_f'] = 
				number_format(35.74 + 0.6215 * $this->decoded_metar['temperature']['temp_f'] 
							  - 35.75 * pow((float)$this->decoded_metar['wind']['mph'], 0.16) 
							  + 0.4275 * $this->decoded_metar['temperature']['temp_f']
							  * pow((float)$this->decoded_metar['wind']['mph'], 0.16));
			$this->decoded_metar['windchill']['windchill_c'] = 
				number_format(13.112 + 0.6215 * $this->decoded_metar['temperature']['temp_c']
							  - 13.37 * pow(($this->decoded_metar['wind']['mph']/1.609), 0.16)
							  + 0.3965 * $this->decoded_metar['temperature']['temp_c']
							  * pow(($this->decoded_metar['wind']['mph']/1.609), 0.16));
		}

		// Compute heat index if temp > 70F
		if (!empty($this->decoded_metar['temperature']['temp_f']) &&
			$this->decoded_metar['temperature']['temp_f'] > 70 &&
			!empty($this->decoded_metar['rel_humidity'])) 
		{
			$WI->status[] = $WI->gmtime() . " Computing Heatindex.";
			$this->decoded_metar['heatindex']['heatindex_f'] =
				number_format(-42.379
							  + 2.04901523 * $this->decoded_metar['temperature']['temp_f']
							  + 10.1433312 * $this->decoded_metar['rel_humidity']
							  - 0.22475541 * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['rel_humidity']
							  - 0.00683783 * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['temperature']['temp_f']
							  - 0.05481717 * $this->decoded_metar['rel_humidity']
										   * $this->decoded_metar['rel_humidity']
							  + 0.00122874 * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['rel_humidity']
							  + 0.00085282 * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['rel_humidity']
										   * $this->decoded_metar['rel_humidity']
							  - 0.00000199 * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['temperature']['temp_f']
										   * $this->decoded_metar['rel_humidity']
										   * $this->decoded_metar['rel_humidity']);

			$this->decoded_metar['heatindex']['heatindex_c'] =
				number_format(($this->decoded_metar['heatindex']['heatindex_f'] - 32) / 1.8);
		}

		// Compute the humidity index
		if (!empty($this->decoded_metar['rel_humidity'])) 
		{
			$WI->status[] = $WI->gmtime() . " Computing Humidty Index.";
			$e = (6.112 * pow(10, 7.5 * $this->decoded_metar['temperature']['temp_c']
				 / (237.7 + $this->decoded_metar['temperature']['temp_c']))
				 * $this->decoded_metar['rel_humidity'] / 100) - 10;
			$this->decoded_metar['humidex']['humidex_c'] =
				number_format($this->decoded_metar['temperature']['temp_c'] + 5/9 * $e, 1);
			$this->decoded_metar['humidex']['humidex_f'] =
				number_format($this->decoded_metar['humidex']['humidex_c'] * 9/5 + 32, 1);
		}
		
		// Humanized the weather conditions
		if (count($this->decoded_metar['weather']) > 0)
		{
			foreach ($this->decoded_metar['weather'] as $count => $weather)
			{
				$conditions = "";
				
				# 12.6.8.e.(2):
				# Tornadoes and waterspouts shall be coded as +FC.
				if ($weather['other'] <> 'FC')
				{
					# Intensity
					switch ($weather['intensity'])
					{
						case '+':  $conditions .= __('Heavy', 'WI')  . ' '; break;
						case '-':  $conditions .= __('Light', 'WI')  . ' '; break;
					}
				}
	
				# Check for Thunderstorm
				switch ($weather['descriptor'])
				{
					# 12.8.6.b.(4):
					# TS can be designated alone or with single or multiple 
					# precipitation codes;
					case 'TS': 
								$conditions .= __('Thunderstorm', 'WI')  . ' '; 
								
								if (strlen(trim($weather['precipitation'] . $weather['obscuration'] . $weather['other'])) > 0)
								{
									$conditions .= '(';
									$conditions .= __('with', 'WI')  . ' ';
								}
								
								break;
				}
				
				# There can be multiple precipitatation codes for one weather
				# event.  
				#
				# IE: Thunderstorm with Snow and Small Hail would be
				# TSSNGS with $weather['precipitation'] = 'SNGS'
				#
				# As a result, we have to explode the string into two character array
				
				foreach (explode("\r\n", chunk_split($weather['precipitation'], 2)) as $pcnt => $precipitation)
				{
					if (strlen(trim($precipitation)) > 0 and $pcnt <> 0)
					{
						$conditions .= __('and', 'WI')  . ' ';
					}
					
					switch (trim($precipitation))
					{
						case 'DZ': 
									# Check for Valid Weather Phenomenoa Qualifier for Drizzle
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(5):
										# FZ can only be designated with FG, DZ, RA
										case 'FZ': $conditions .= __('Freezing', 'WI')  . ' '; break;
									}
	
									$conditions .= __('Drizzle', 'WI')  . ' '; 
									
									break;
									
						case 'RA': 
									# Check for Valid Weather Phenomenoa Qualifier for Rain
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(5):
										# FZ can only be designated with FG, DZ, RA
										case 'FZ': $conditions .= __('Freezing', 'WI')  . ' '; break;
									}
	
									$conditions .= __('Rain', 'WI')  . ' '; 
	
									# Check for Valid Weather Phenomenoa Qualifier for Snow
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(3): SH only with RA, SN, PL, GS, and GR
										# Changed to Falling since Snow Showers is stupid
										case 'SH': $conditions .= __('Showers', 'WI')  . ' ';
									}
	
									break;
									
						case 'SN': 
									# Check for Valid Weather Phenomenoa Qualifier for Snow
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(2):
										# DR & BL only with Dust (DU), Sand (SA), or Snow (SN)
										case 'DR': $conditions .= __('Low Drifting', 'WI')  . ' '; break;
										case 'BL': $conditions .= __('Blowing', 'WI')  . ' '; break;
				
										# 12.8.6.b.(3): SH only with RA, SN, PL, GS, and GR
										# Changed to Falling since Snow Showers is stupid
										case 'SH': $conditions .= __('Falling', 'WI')  . ' ';
									}
	
									$conditions .= __('Snow', 'WI')  . ' ';
									break;
									
						case 'SG':	$conditions .= __('Snow Grains', 'WI') . ' '; break;
						case 'IC':	$conditions .= __('Ice Crystals', 'WI') . ' '; break;
						case 'PL': 
									$conditions .= __('Ice Pellets', 'WI') . ' '; 
									
									# Check for Valid Weather Phenomenoa Qualifier for Snow
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(3): SH only with RA, SN, PL, GS, and GR
										# Changed to Falling since Snow Showers is stupid
										case 'SH': $conditions .= __('Showers', 'WI') . ' ';
									}
									
									break;
									
						case 'GR': 
									$conditions .= __('Hail', 'WI') . ' '; 
	
									# Check for Valid Weather Phenomenoa Qualifier for Snow
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(3): SH only with RA, SN, PL, GS, and GR
										# Changed to Falling since Snow Showers is stupid
										case 'SH': $conditions .= __('Showers', 'WI') . ' ';
									}
	
									break;
									
						case 'GS': 
									$conditions .= __('Small Hail', 'WI') . ' '; 
									
									# Check for Valid Weather Phenomenoa Qualifier for Snow
									switch ($weather['descriptor'])
									{
										# 12.8.6.b.(3): SH only with RA, SN, PL, GS, and GR
										# Changed to Falling since Snow Showers is stupid
										case 'SH': $conditions .= __('Showers', 'WI') . ' ';
									}	
									break;
									
						case 'UP':	$conditions .= __('Unknown Precipitation', 'WI') . ' '; break;
					}
				}
				
				switch ($weather['obscuration'])
				{
					case 'BR':	$conditions .= __('Mist', 'WI') . ' '; break;
					case 'FG': 
								# Check for Valid Weather Phenomenoa Qualifier for Fog
								switch ($weather['descriptor'])
								{
									# 12.8.6.b.(1): MI/PR/BC only with Fog (FG)
									case 'MI': $conditions .= __('Shallow (Ground)', 'WI') . ' '; break;
									case 'PR': $conditions .= __('Partial', 'WI') . ' '; break;
									case 'BC': $conditions .= __('Patches', 'WI') . ' '; break;
									# 12.8.6.b.(5): FZ can only be designated with FG, DZ, RA
									case 'FZ': $conditions .= __('Freezing', 'WI') . ' '; break;
								}					
								$conditions .= __('Fog', 'WI') . ' '; 
								break;
	
					case 'FU':	$conditions .= __('Smoke', 'WI') . ' '; break;
					case 'VA':	$conditions .= __('Volcanic Ash', 'WI') . ' '; break;
					case 'DU': 
								# Check for Valid Weather Phenomenoa Qualifier for Snow
								switch ($weather['descriptor'])
								{
									# 12.8.6.b.(2):
									# DR & BL only with Dust (DU), Sand (SA), or Snow (SN)
									case 'DR': $conditions .= __('Low Drifting', 'WI') . ' '; break;
									case 'BL': $conditions .= __('Blowing', 'WI') . ' '; break;
								}								
	
								$conditions .= __('Dust', 'WI') . ' ';
								break;
								
					case 'SA': 
								# Check for Valid Weather Phenomenoa Qualifier for Snow
								switch ($weather['descriptor'])
								{
									# 12.8.6.b.(2):
									# DR & BL only with Dust (DU), Sand (SA), or Snow (SN)
									case 'DR': $conditions .= __('Low Drifting', 'WI') . ' '; break;
									case 'BL': $conditions .= __('Blowing', 'WI') . ' '; break;
								}								
	
								$conditions .= __('Sand', 'WI') . ' '; 
								break;
								
					case 'HZ':	$conditions .= __('Haze', 'WI') . ' '; break;
					
					# Must have a descriptor of BL as per 12.8.6.d.(3)
					case 'PY': 
								# 12.8.6.d.(3):
								# Spray (PY) must use the descriptor of BL
								if ($weather['descriptor'] = 'BL') 
								{
									$conditions .= __('Blowing', 'WI') . ' ';
									$conditions .= __('Spray', 'WI') . ' '; 
								}
								break;
				}
				
				switch ($weather['other'])
				{
					case 'PO':	$conditions .= __('Well Developed Dust/Sand Whirls', 'WI') . ' '; break;
					case 'SQ':	$conditions .= __('Squalls', 'WI') . ' '; break;
					
					# 12.6.8.e.(2):
					# Tornados are coded as +FC, and Funnel Clouds are only FC
					case 'FC':	
								if ($weather['intensity'] = '+')
								{
									$conditions .= __('Tornado', 'WI') . ' ';
								}
								else						   
								{
									$conditions .= __('Funnel Cloud', 'WI') . ' '; 
								}
								break;
								
					case 'SS':	$conditions .= __('Sandstorm', 'WI') . ' '; break;
					case 'DS':	$conditions .= __('Duststorm', 'WI') . ' '; break;
				}
	
				# Check for Thunderstorm
				switch ($weather['descriptor'])
				{
					# 12.8.6.b.(4):
					# TS can be designated alone or with single or multiple 
					# precipitation codes;
					case 'TS': 
								if (strlen(trim($weather['precipitation'] . $weather['obscuration'] . $weather['other'])) > 0)
								{
									$conditions = substr($conditions,0,strlen(trim($conditions))-1);
									$conditions .= ') ';
								}
								
								break;
				}
	
				# Proximity of Weather
				if ($weather['proximity'] <> '') $conditions .= __('in the vicinity', 'WI')  . ' ';
				
				# Save translated text into tag "humanized"
				$this->decoded_metar['weather'][$count]['humanized'] = $conditions;
			}
		}
		
		// Humanize the cloud conditions
		if (count($this->decoded_metar['clouds']) > 0)
		{
			foreach ($this->decoded_metar['clouds'] as $count => $cloud)
			{	
				switch($cloud['condition'])
				{
					case 'CAVOK': 
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Cloud and Visibility OK', 'WI');
						break;					  
					case 'FEW':
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Few Clouds', 'WI');
						break;
					case 'SCT':
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Scattered Clouds', 'WI');
						break;
					case 'BKN':
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Broken Clouds', 'WI');
						break;
					case 'OVC':
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Overcast', 'WI');
						break;
					case 'VV':
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Indefinite Ceiling', 'WI');
						break;
					// SKC, CLR, NSC
					default:
						$this->decoded_metar['clouds'][$count]['humanized'] = __('Clear Skies', 'WI');
						break;
				}
			}
		}

		// Determine the Icon Name to display
		$cloud = $this->decoded_metar['clouds'][count( $this->decoded_metar['clouds'] ) - 1]['condition'];

		// Check for Thunderstorm Activity (overrides all other conditions)
		if ($this->decoded_metar['weather'][0]['descriptor'] == 'TS')
		{
			$condition = $this->decoded_metar['weather'][0]['descriptor'];
		}
		else
		{
			$condition = $this->decoded_metar['weather'][0]['intensity'] . $this->decoded_metar['weather'][0]['obscuration'] . substr($this->decoded_metar['weather'][0]['precipitation'],0,2);
		}

		if (array_key_exists ($cloud . $condition, $WI->icon_packs[$WI->settings['icon_pack']]['conditions']))
		{
			$icon = $WI->icon_packs[$WI->settings['icon_pack']]['conditions'][$cloud . $condition];
			$this->decoded_metar['icon'] = $this->day_night(strtolower($WI->icon_packs[$WI->settings['icon_pack']]['icons'][$icon])=='true') . $icon;
			$WI->status[] .= $WI->gmtime() . ' key is ' . $cloud.$condition . ', and resulting icon is ' . $this->decoded_metar['icon'];
		}
		else if (array_key_exists ($condition, $WI->icon_packs[$WI->settings['icon_pack']]['conditions']))
		{
			$icon = $WI->icon_packs[$WI->settings['icon_pack']]['conditions'][$condition];
			$this->decoded_metar['icon'] = $this->day_night(strtolower($WI->icon_packs[$WI->settings['icon_pack']]['icons'][$icon])=='true') . $icon;
			$WI->status[] .= $WI->gmtime() . ' key is ' . $condition . ', and resulting icon is ' . $this->decoded_metar['icon'];
		}
		else if (array_key_exists ($cloud, $WI->icon_packs[$WI->settings['icon_pack']]['conditions']))
		{
		    $icon = $WI->icon_packs[$WI->settings['icon_pack']]['conditions'][$cloud];
			$this->decoded_metar['icon'] = $this->day_night(strtolower($WI->icon_packs[$WI->settings['icon_pack']]['icons'][$icon])=='true') . $icon;
			$WI->status[] .= $WI->gmtime() . ' key is ' . $cloud . ', and resulting icon is ' . $this->decoded_metar['icon'];
		}
		else if (array_key_exists ('noimage',  $WI->icon_packs[$WI->settings['icon_pack']]['icons']))
		{
			$this->decoded_metar['icon'] = 'noimage';
			$WI->status[] .= $WI->gmtime() . ' using noimage image.';
		}
		else
		{
			$this->decoded_metar['icon'] = $this->day_night(strtolower(current($WI->icon_packs[$WI->settings['icon_pack']]['icons']))=='true') . current(array_keys($WI->icon_packs[$WI->settings['icon_pack']]['icons']));
			$WI->status[] .= $WI->gmtime() . ' no noimage, no icon for conditions('.$condition.')+cloud('.$cloud.') or cloud/condition byitself, default to first icon in package.';
		}
	}

	function show_plus ($value)
	{
		if (intval($value) > 0)
		{
			return "+";
		}
		else
		{
			return "";
		}
	}

	function show_gmt ($value)
	{
		$gmt = $this->show_plus ($value);
		if (intval($value) != 0)
		{
			$gmt .=	intval($value);
		}
		return $gmt;
	}
	
	function get_temp ()
	{
		global $WI;

		$text = __('<span class="weather_title">Temperature:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$values = array( number_format($this->decoded_metar['temperature']['temp_'.strtolower($WI->settings['units']['temp'])], 0),
						 __('&#176;' . $WI->settings['units']['temp'], 'WI') );

		return $this->before . preg_replace($search, $values, $text) . $this->after;
	}

	function get_windchill ()
	{
		global $WI;

		$text = __('<span class="weather_title">Wind Chill:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$values = array( number_format($this->decoded_metar['windchill']['windchill_'.strtolower($WI->settings['units']['temp'])], 0),
						 __('&#176;' . $WI->settings['units']['temp'], 'WI') );

		if (($this->decoded_metar['temperature']['temp_f'] < 51
			&& $this->decoded_metar['wind']['mph'] > 3))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}	

	function get_heatindex ()
	{
		global $WI;

		$text = __('<span class="weather_title">Heat Index:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$values = array( number_format($this->decoded_metar['heatindex']['heatindex_'.strtolower($WI->settings['units']['temp'])], 0),
						 __('&#176;' . $WI->settings['units']['temp'], 'WI') );

		if (($this->decoded_metar['temperature']['temp_f'] >= 80
			&& !empty($this->decoded_metar['rel_humidity'])))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}

	function get_humidex ()
	{
		global $WI;

		$text = __('<span class="weather_title">Humidex:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$values = array( number_format($this->decoded_metar['humidex']['humidex_'.strtolower($WI->settings['units']['temp'])], 0),
						 __('&#176;' . $WI->settings['units']['temp'], 'WI') );

		if (($this->decoded_metar['temperature']['temp_f'] > 79
		 && $this->decoded_metar['rel_humidity'] > 39))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}
		
	function get_humidity()
	{
		global $WI;

		$text = __('<span class="weather_title">Humidity:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");
		
		$values = array( $this->decoded_metar['rel_humidity'],
						 __('%', 'WI') );
						 
		if ((!empty($this->decoded_metar['rel_humidity'])))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}

	function get_wind ()
	{
		global $WI;

		$text = __('<span class="weather_title">Wind:</span> <span class="weather_info">#dir# #at# #value# #units# #gustingto# #gust_value# #gust_units#</span>', 'WI');
		
		$search = array("/#dir#/",
						"/#at#/",
						"/#value#/",
						"/#units#/",
						"/#gustingto#/",
						"/#gust_value#/",
						"/#gust_units#/",
						"/\s\s+/");
		
		$gustingto = '';
		$gust_value = '';
		$gust_units = '';		
		if ($this->decoded_metar['wind']['gust'])
		{
			$gustingto = __('gusting to', 'WI');
			$gust_value = number_format($this->decoded_metar['wind']['gust'][strtolower($WI->settings['units']['wind'])], 0);
			$gust_units = __($WI->settings['units']['wind'], 'WI');
		}
	
		if ( number_format($this->decoded_metar['wind'][strtolower($WI->settings['units']['wind'])], 0) > 0 )
		{
			$values = array( $this->decoded_metar['wind']['dir'],
						 __('at', 'WI'),
						 number_format($this->decoded_metar['wind'][strtolower($WI->settings['units']['wind'])], 0),
						 __($WI->settings['units']['wind'], 'WI'),
						 $gustingto,
						 $gust_value,
						 $gust_units,
						 '');
		}
		else
		{
			$values = array( __('Calm', 'WI'), '', '', '', '', '', '', '' );
		}
	
		return $this->before . preg_replace($search, $values, $text) . $this->after;
	}

	function get_dewpoint ()
	{
		global $WI;

		$text = __('<span class="weather_title">Dew Point:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$values = array( number_format($this->decoded_metar['temperature']['dew_'.strtolower($WI->settings['units']['temp'])], 0), 
						 __('&#176;' . $WI->settings['units']['temp'], 'WI') );

		if ((!empty($this->decoded_metar['temperature']['dew_'.strtolower($WI->settings['units']['temp'])])))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}

	function get_altimeter ()
	{
		global $WI;

		$text = __('<span class="weather_title">Barometer:</span> <span class="weather_info">#value# #units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$values = array( $this->decoded_metar['altimeter'][strtolower($WI->settings['units']['pressure'])],
						 __($WI->settings['units']['pressure'], 'WI') );

		if ((!empty($this->decoded_metar['altimeter'][strtolower($WI->settings['units']['pressure'])])))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}
	
	function get_conditions_loop()
	{
		$conditions = '';
		if (count($this->decoded_metar['weather']) > 0)
		{
			foreach ($this->decoded_metar['weather'] as $count => $weather)
			{
				if ($count <> 0) $conditions .= __('and', 'WI') . ' ';
				$conditions .= $weather['humanized'];
			}
		}
		
		return $conditions;
	}

	function get_conditions ()
	{
		global $WI;

		$text = __('<span class="weather_title">Conditions:</span> <span class="weather_info">#value#</span>', 'WI');
		
		$conditions = $this->get_conditions_loop();
		
		if (strlen(trim($conditions)) > 0)
		{
			$search = array( "/#value#/" );
			$values = array( $conditions );

			return  $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}
	
	function get_visibility ()
	{
		global $WI;

		$text = __('<span class="weather_title">Visibility:</span> <span class="weather_info">#value##units#</span>', 'WI');

		$search = array("/#value#/",
						"/#units#/");

		$units = __($WI->settings['units']['visibility'], 'WI');

		// This is a dirty (and ugly) little hack by Viper to add a space if we're using miles (it doesn't look right if we don't do this)
		// And yes, this is purposely done for English only in order to allow translators to do as they wish	
		if ($units == 'miles')
		{
			$units = ' ' . $units;
		}

		$values = array( number_format($this->decoded_metar['visibility'][0][strtolower($WI->settings['units']['visibility'])], 0), $units);

		if ((!empty($this->decoded_metar['visibility'][0][strtolower($WI->settings['units']['visibility'])])))
		{
			return $this->before . preg_replace($search, $values, $text) . $this->after;
		}
	}

	function get_updatetime ()
	{
		global $WI;

		$text = __('<span class="weather_title">Updated:</span> <span class="weather_info" title="#date# #GMT_OFFSET#">#time# #GMT_OFFSET#</span>', 'WI');

		$search = array("/#time#/", "/#date#/", "/#GMT_OFFSET#/");

		// Okay, so for the moment, we are forcing time only ... will do settings to select again later.
		$timeformat = $WI->settings['default_time'];
		$dateformat = $WI->settings['default_date'];

		// Ensure we have a dateformat before trying to figure out the date ...
		if (strlen (trim($dateformat)) > 0)
		{	
			$values = array( 
				gmdate( $timeformat, intval($this->decoded_metar['time']['update']) + (intval($this->gmt_offset) * 3600) ),
				gmdate( $dateformat, intval($this->decoded_metar['time']['update']) + (intval($this->gmt_offset) * 3600) ),
				__('GMT', 'WI') . $this->show_gmt($this->gmt_offset)
			);

			if (!empty($this->decoded_metar['time']['update']))
			{
				return $this->before . preg_replace($search, $values, $text) . $this->after;
			}
		}
	}
	
	function get_simple_clouds()
	{
		global $WI;

		$text = __('<span class="weather_title">Clouds:</span> <span class="weather_info">#value#</span>', 'WI');
		
		$clouds = $this->decoded_metar['clouds'][count($this->decoded_metar['clouds'])-1]['humanized'];
		
		if (strlen(trim($clouds)) > 0)
			return $this->before . preg_replace("/#value#/", $clouds, $text) . $this->after;
		
		return '';
	}
	
	function get_detailed_clouds ()
	{
		global $WI;

		$text = __('<span class="weather_title">Clouds:</span> <span class="weather_info">#value#</span>', 'WI');

		$clouds = "";
		if (count($this->decoded_metar['clouds']) > 0)
		{
			foreach ($this->decoded_metar['clouds'] as $count => $cloud)
			{	
				if ($count <> 0) $clouds .= __('and', 'WI')  . ' ';
				
				$clouds .= $cloud['humanized'] . ' ';
				
				if (!empty($cloud['cumulus']))
				{
					switch($cloud['cumulus'])
					{
						case 'CB':   $clouds .= '(' . __('cumulonimbus', 'WI') . ') '; break;
						case 'TCU':  $clouds .= '(' . __('towering cumulus', 'WI') . ') '; break;
					}
				}
				
				if ($cloud['condition'] != 'CAVOK'
				 && $cloud['condition'] != 'SKC'
				 && $cloud['condition'] != 'CLR'
				 && $cloud['condition'] != 'NSC'
				 && $cloud['condition'] != 'VV')
				{
					if ($cloud[strtolower($WI->settings['units']['clouds'])] != 'nil')
					{
						$clouds .= __('at', 'WI') . ' ' . $cloud[strtolower($WI->settings['units']['clouds'])] . $WI->settings['units']['clouds'] . ' ';
					}
					else
					{
						$clouds .= __('with unknown cloud base', 'WI') . ' ';
					}
				}
				else if ($cloud['condition'] == 'VV')
				{
					$clouds .= '(' . __('vertical visibility', 'WI') . ': ' . $cloud[strtolower($WI->settings['units']['clouds'])] . $WI->settings['units']['clouds'] . ') ';
				}
			}
		}
		
		if (strlen(trim($clouds)) > 0)
			return $this->before . preg_replace("/#value#/", $clouds, $text) . $this->after;
			
		return '';
	}

	function get_icon()
	{
		global $WI;

		$src = WIdirWeb . "/ipkfile.php?pack=" . $WI->settings['icon_pack'] . '&amp;icon=' . $this->decoded_metar['icon'] . '&amp;type=' . $WI->icon_packs[$WI->settings['icon_pack']]['formats']['format'][$WI->settings['icon_ext']];

		// Change the $before tag for this element ... append
		// class="wicon" to it.
		$wicon_before = $this->before;

		if ( substr($wicon_before, -1,1) == '>')
		{
			$wicon_before = substr($wicon_before,0,strlen($wicon_before)-1) . ' class="wicon">';
		}

		$WI->status[] = $WI->gmtime() . " wicon_before is " . htmlentities ($wicon_before);

		// If the user is using IE5 or IE6 and we're using PNG images, do an IE hack because IE is retarded.
		if (preg_match('/msie\s(5\.[5-9]|6\.[0-9]*).*(win)/i', $_SERVER['HTTP_USER_AGENT']) && strtolower(substr($src, -4)) == '.png')
		{
		
			// Get image Size (requires GD support in PHP)
			if (function_exists('getimagesize'))
			{
				$tar = new Archive_Tar(WIicondir . $WI->settings['icon_pack'] . '.ipk');
				$img = $tar->extractInString($this->decoded_metar['icon'] . $WI->icon_packs[$WI->settings['icon_pack']]['formats']['format'][$WI->settings['icon_ext']]);

				// Should never be empty in a properly designed image pack.
				if (!empty($img))
				{
					$width = imagesx($img);
					$height = imagesy($img);
				}
			}
			else
			{
				$width = 80;
				$height = 80;
			}

			return $wicon_before . '<span style="height:'.$height.'px; width:'.$width.'px; display:inline-block; position:relative; filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src=\'' . $src . '\', sizingMethod=\'scale\');" title="' . $this->get_status() . '"></span>' . $this->after;
		}
		else
		{
			return $wicon_before . '<img src="' . $src . '"' . $style . ' title="' . $this->get_status() . '" alt="' . $this->get_status() . '" />' . $this->after;
		}
	}

	function get_status()
	{
		$conditions = trim($this->get_conditions_loop());

		if (strlen($conditions) > 0)
		{
			return $conditions;
		}
		else
		{
			if (count($this->decoded_metar['clouds']) > 0)
			{
				return $this->decoded_metar['clouds'][count($this->decoded_metar['clouds'])-1]['humanized'];
			}

			return '';
		}
	}
	
	function is_daylight_time($time) {
		list($dom, $dow, $month, $hour, $min) = explode(":", date("d:w:m:H:i", $time));

		if ($month > 4 && $month < 10) {
			$retval = 1;		 // May thru September
		} elseif ($month == 4 && $dom > 7) {
			$retval = 1;		 // After first week in April
		} elseif ($month == 4 && $dom <= 7 && $dow == 0 && $hour >= 2) {
			$retval = 1;		 // After 2am on first Sunday ($dow=0) in April
		} elseif ($month == 4 && $dom <= 7 && $dow != 0 && ($dom-$dow > 0)) {
			$retval = 1;		 // After Sunday of first week in April
		} elseif ($month == 10 && $dom < 25) {
			$retval = 1;		 // Before last week of October
		} elseif ($month == 10 && $dom >= 25 && $dow == 0 && $hour < 2) {
			$retval = 1;		 // Before 2am on last Sunday in October
		} elseif ($month == 10 && $dom >= 25 && $dow != 0 && ($dom-24-$dow < 1) ) {
			$retval = 1;		 // Before Sunday of last week in October
		} else {
			$retval = 0;
		}
		return $retval;
	}

	function sunrise_sunset($latitude, $longitude, $timezone, $dst=TRUE) {
		$yday = date("z");
		$mon = date("n");
		$mday = date("j");
		$year = date("Y");

		// Check to see if we are in the time of year that DST is in effect
		// and check to see if location observes DST.
		$DST = $this->is_daylight_time(date("U")) && $dst;
		
		if ($DST) {$timezone = ($timezone + 1);}
		if ($timezone == "13") {$timezone = "-11";}

		$A = 1.5708;
		$B = 3.14159;
		$C = 4.71239;
		$D = 6.28319;
		$E = 0.0174533 * $latitude;
		$F = 0.0174533 * $longitude;
		$G = 0.261799  * $timezone;

		// For astronomical twilight, use R = -.309017
		// For	 nautical twilight, use R = -.207912
		// For		civil twilight, use R = -.104528
		// For	 sunrise or sunset, use R = -.0145439
		$R = -.0145439;

		for ($i = 0; $i < 2; $i++) {

			if (!$i) {$J =  $A;  $type = "rise";}	// calculate sunrise
				else  {$J =  $C;  $type = "set"; }   // calculate sunset

			$K = $yday + (($J - $F) / $D);
			$L = ($K * .017202) - .0574039;			  // Solar Mean Anomoly
			$M = $L + .0334405 * sin($L);				// Solar True Longitude
			$M += 4.93289 + (3.49066E-04) * sin(2 * $L);

			// Quadrant Determination
			if ($D == 0) {break;}
			while ($M < 0)   {$M = ($M + $D);}
			while ($M >= $D) {$M = ($M - $D);}

			if (($M / $A) - intval($M / $A) == 0) {$M += 4.84814E-06;}
			$P = sin($M) / cos($M);				   // Solar Right Ascension
			$P = atan2(.91746 * $P, 1);

			// Quadrant Adjustment
			if ($M > $C) {$P += $D;}
			else {if ($M > $A) {$P += $B;} }

			$Q = .39782 * sin($M);			// Solar Declination
			$Q = $Q / sqrt(-$Q * $Q + 1);	 // This is how the original author wrote it!
			$Q = atan2($Q, 1);

			$S = $R - (sin($Q) * sin($E));
			$S = $S / (cos($Q) * cos($E));

			if (abs($S) > 1) {break;}	 // Null phenomenon

			$S = $S / sqrt(-$S * $S + 1);
			$S = $A - atan2($S, 1);

			if ($type == 'rise') {$S = $D - $S ;}

			$T = $S + $P - 0.0172028 * $K - 1.73364; // Local apparent time
			$U = $T - $F;							// Universal timer
			$V = $U + $G;							// Wall clock time

			// Quadrant Determination
			if ($D == 0) {break;}
			while ($V < 0)   {$V = ($V + $D);}
			while ($V >= $D) {$V = ($V - $D);}
			$V = $V * 3.81972;

			$hour = intval($V);
			$min  = intval((($V - $hour) * 60) + 0.5);

			$this->decoded_metar['time']['sun'.$type] = mktime($hour,$min,0,$mon,$mday,$year);
		}
	}

	function day_night($night_icon_available) {
		global $WI;

		if ($WI->settings['usetime'] == 'metartime')
		{
			$ctime = intval($this->decoded_metar['time']['update']);
		}
		else
		{
			$ctime = time();
		}

		$now = intval(gmdate("H", $ctime)) + intval($this->gmt_offset) + (intval(gmdate("i", $ctime))/60);

		// Ensure hour is within the 0-24 range
		if ($now < 0) $now += 24;
		else if ($now > 24) $now -= 24;

		if ((isset($this->decoded_metar['time']['sunrise'])) && (isset($this->decoded_metar['time']['sunset']))) {
			$sunrise = intval(date("H", $this->decoded_metar['time']['sunrise'])) + (intval(date("i", $this->decoded_metar['time']['sunrise']))/60);
			$sunset  = intval(date("H", $this->decoded_metar['time']['sunset'])) + (intval(date("i", $this->decoded_metar['time']['sunset']))/60);

			// Ensure sunrise/sunset hour within 0-24
			if ($sunrise < 0)  $sunrise = $sunrise + 24;
			if ($sunrise > 24) $sunrise = $sunrise - 24;
			if ($sunset < 0)   $sunset = $sunset + 24;
			if ($sunset > 24)  $sunset = $sunset - 24;

			$WI->status[] = $WI->gmtime() . " Sunrise Hour is " . $sunrise;
			$WI->status[] = $WI->gmtime() . " Sunset Hour is " . $sunset;
			$WI->status[] = $WI->gmtime() . " Current Hour is " . $now;
		} else {
			$sunrise = $WI->settings['sunrise'];
			$sunset  = $WI->settings['sunset'];
		}

		// if night, icon maybe different
		if (($now >= $sunset && $now <= $sunrise)
			||((($now >= $sunset && $now <= 24)
				|| ($now <= $sunrise && $now >= 0))
				&& $sunset > $sunrise))
		{
			$WI->status[] = $WI->gmtime() . " Night time, " . (($night_icon_available)?"icon available.":"no icon available.");
			return ($night_icon_available ? 'n_': '');
		}

		return '';
	}

	function get_sunrise ()
	{
		global $WI;

		$text = __('<span class="weather_title">Sunrise:</span> <span class="weather_info" title="#date# #GMT_OFFSET#">#time# #GMT_OFFSET#</span>', 'WI');
		$search = array("/#time#/", "/#date#/", "/#GMT_OFFSET#/");

		$sunrise = $this->decoded_metar['time']['sunrise'];

		$timeformat = $WI->settings['default_time'];
		$dateformat = $WI->settings['default_date'];

		// Ensure we have a dateformat before trying to figure out the date ...
		if (strlen (trim($dateformat)) > 0)
		{
			// replaced gmdate with just date.  time returned is already
			// in correct time zone, and using gmdate adjusts the timestamp
			// by the timezone of system (ouch).
			$values = array( 
				date( $timeformat, $sunrise),
				date( $dateformat, $sunrise),
				__('GMT', 'WI') . $this->show_gmt($this->gmt_offset)
			);

			if (!empty($sunrise))
			{
				return $this->before . preg_replace($search, $values, $text) . $this->after;
			}
		}
	}

	function get_sunset ()
	{
		global $WI;

		$text = __('<span class="weather_title">Sunset:</span> <span class="weather_info" title="#date# #GMT_OFFSET#">#time# #GMT_OFFSET#</span>', 'WI');
		$search = array("/#time#/", "/#date#/", "/#GMT_OFFSET#/");

		$sunset = $this->decoded_metar['time']['sunset'];
		$timeformat = $WI->settings['default_time'];
		$dateformat = $WI->settings['default_date'];

		// Ensure we have a dateformat before trying to figure out the date ...
		if (strlen (trim($dateformat)) > 0)
		{
			// replaced gmdate with just date.  time returned is already
			// in correct time zone, and using gmdate adjusts the timestamp
			// by the timezone of system (ouch).
			$values = array( 
				date( $timeformat, $sunset ),
				date( $dateformat, $sunset ),
				__('GMT', 'WI') . $this->show_gmt($this->gmt_offset)
			);

			if (!empty($sunset))
			{
				return $this->before . preg_replace($search, $values, $text) . $this->after;
			}
		}
	}
}

?>
